<?php

/**
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

declare(strict_types=1);

namespace Chill\MainBundle\Controller;

use Chill\MainBundle\Entity\Notification;
use Chill\MainBundle\Entity\User;
use Chill\MainBundle\Security\Authorization\NotificationVoter;
use Doctrine\ORM\EntityManagerInterface;
use RuntimeException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Component\Security\Core\Security;
use UnexpectedValueException;

/**
 * @Route("/api/1.0/main/notification")
 */
class NotificationApiController
{
    private EntityManagerInterface $entityManager;

    private Security $security;

    public function __construct(EntityManagerInterface $entityManager, Security $security)
    {
        $this->entityManager = $entityManager;
        $this->security = $security;
    }

    /**
     * @Route("/{id}/mark/read", name="chill_api_main_notification_mark_read", methods={"POST"})
     */
    public function markAsRead(Notification $notification): JsonResponse
    {
        return $this->markAs('read', $notification);
    }

    /**
     * @Route("/{id}/mark/unread", name="chill_api_main_notification_mark_unread", methods={"POST"})
     */
    public function markAsUnread(Notification $notification): JsonResponse
    {
        return $this->markAs('unread', $notification);
    }

    private function markAs(string $target, Notification $notification): JsonResponse
    {
        if (!$this->security->isGranted(NotificationVoter::NOTIFICATION_TOGGLE_READ_STATUS, $notification)) {
            throw new AccessDeniedException('Not allowed to toggle read status of notification');
        }

        $user = $this->security->getUser();

        if (!$user instanceof User) {
            throw new RuntimeException('not possible to mark as read by this user');
        }

        switch ($target) {
            case 'read':
                $notification->markAsReadBy($user);

                break;

            case 'unread':
                $notification->markAsUnreadBy($user);

                break;

            default:
                throw new UnexpectedValueException("target not supported: {$target}");
        }

        $this->entityManager->flush();

        return new JsonResponse(null, JsonResponse::HTTP_ACCEPTED, [], false);
    }
}
