<?php

/**
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

declare(strict_types=1);

namespace Chill\MainBundle\Form\Type\DataTransformer;

use Chill\MainBundle\Entity\User;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception\TransformationFailedException;
use Symfony\Component\Serializer\Normalizer\AbstractNormalizer;
use Symfony\Component\Serializer\Normalizer\DenormalizerInterface;
use Symfony\Component\Serializer\SerializerInterface;
use function array_key_exists;

class UserToJsonTransformer implements DataTransformerInterface
{
    private DenormalizerInterface $denormalizer;

    private bool $multiple;

    private SerializerInterface $serializer;

    public function __construct(DenormalizerInterface $denormalizer, SerializerInterface $serializer, bool $multiple)
    {
        $this->denormalizer = $denormalizer;
        $this->serializer = $serializer;
        $this->multiple = $multiple;
    }

    public function reverseTransform($value)
    {
        if ($this->multiple) {
            return array_map(
                function ($item) { return $this->denormalizeOne($item); },
                json_decode($value, true)
            );
        }

        return $this->denormalizeOne(json_decode($value, true));
    }

    /**
     * @param User|User[] $value
     */
    public function transform($value): string
    {
        if (null === $value) {
            return $this->multiple ? 'null' : '[]';
        }

        return $this->serializer->serialize($value, 'json', [
            AbstractNormalizer::GROUPS => ['read'],
        ]);
    }

    private function denormalizeOne(array $item): User
    {
        if (!array_key_exists('type', $item)) {
            throw new TransformationFailedException('the key "type" is missing on element');
        }

        if (!array_key_exists('id', $item)) {
            throw new TransformationFailedException('the key "id" is missing on element');
        }

        return
            $this->denormalizer->denormalize(
                ['type' => $item['type'], 'id' => $item['id']],
                User::class,
                'json',
                [AbstractNormalizer::GROUPS => ['read']],
            );
    }
}
